package com.qmth.boot.core.concurrent.service.impl;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import javax.annotation.PreDestroy;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class MemoryConcurrentService implements ConcurrentService {

    private final Map<String, ReentrantLock> lockMap;

    private final Map<String, ReentrantReadWriteLock> readWriteMap;

    public MemoryConcurrentService() {
        this.lockMap = new HashMap<>();
        this.readWriteMap = new HashMap<>();
    }

    @PreDestroy
    public void close() {
        this.lockMap.clear();
        this.readWriteMap.clear();
    }

    private ReentrantLock lock(String name) {
        ReentrantLock lock = lockMap.get(name);
        if (lock == null) {
            synchronized (lockMap) {
                lock = lockMap.computeIfAbsent(name, key -> new ReentrantLock());
            }
        }
        return lock;
    }

    private ReentrantReadWriteLock readWriteLock(String name) {
        ReentrantReadWriteLock lock = readWriteMap.get(name);
        if (lock == null) {
            synchronized (readWriteMap) {
                lock = readWriteMap.computeIfAbsent(name, key -> new ReentrantReadWriteLock());
            }
        }
        return lock;
    }

    @Override
    public Lock getLock(String name) {
        return lock(name);
    }

    @Override
    public boolean isLocked(String name) {
        return lock(name).isLocked();
    }

    @Override
    public ReadWriteLock getReadWriteLock(String name) {
        return readWriteLock(name);
    }

    @Override
    public boolean isReadLocked(String name) {
        return readWriteLock(name).getReadLockCount() > 0;
    }

    @Override
    public boolean isWriteLocked(String name) {
        return readWriteLock(name).isWriteLocked();
    }
}
