package com.qmth.boot.api.interceptor;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.config.AacConfig;
import com.qmth.boot.api.config.ApiProperties;
import com.qmth.boot.api.constant.ApiConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

public abstract class AbstractInterceptor extends HandlerInterceptorAdapter implements ApiConstant {

    protected final Logger log = LoggerFactory.getLogger(this.getClass());

    private Map<Object, AacConfig> configMap = new HashMap<>();

    protected AacConfig getAacConfig(ApiProperties apiProperties, HttpServletRequest request, Object handler) {
        AacConfig aacConfig = getAacConfig(request);
        if (aacConfig == null) {
            aacConfig = configMap.computeIfAbsent(handler, k -> {
                AacConfig ac = new AacConfig(apiProperties);
                if (handler instanceof HandlerMethod) {
                    HandlerMethod method = (HandlerMethod) handler;
                    ac.merge(new AacConfig(method.getBeanType().getAnnotation(Aac.class)));
                    ac.merge(new AacConfig(method.getMethodAnnotation(Aac.class)));
                }
                return ac;
            });
            request.setAttribute(ATTRIBUTE_AAC_CONFIG, apiProperties);
        }
        return aacConfig;
    }

    private AacConfig getAacConfig(HttpServletRequest request) {
        Object obj = request.getAttribute(ATTRIBUTE_AAC_CONFIG);
        if (obj instanceof AacConfig) {
            return (AacConfig) obj;
        } else {
            return null;
        }
    }

}
